@@ -1,18 +1,15 @@
+#ifndef IDEFINES_H
+# define IDEFINES_H
+
 #include <config.h>
 
 /* #define DEBUG Don't even think defining it:) */
 
 #include <mhash.h>
-#ifndef LIBMCRYPT_22
-# include <mcrypt.h>
-#else
-# include <mcrypt-nm.h>
-#endif
+#include <mcrypt.h>
 
-#ifndef LIBMCRYPT_22
-# if MCRYPT_API_VERSION < 20000222
-#  error "Your mcrypt.h header file is older than the library"
-# endif
+#if MCRYPT_API_VERSION < 20000222
+# error "Your mcrypt.h header file is older than the library"
 #endif
 
 #ifdef STDC_HEADERS
@@ -21,7 +18,6 @@
 # include <stdio.h>
 #endif
 
-
 #ifdef HAVE_CTYPE_H
 # include <ctype.h>
 #endif
@@ -39,10 +35,10 @@
 
 #ifdef BZIP2
 # define ZIP
-#endif
-
-#ifdef GZIP
-# define ZIP
+#else
+# ifdef GZIP
+#  define ZIP
+# endif
 #endif
 
 #ifdef HAVE_SYS_WAIT_H
@@ -192,3 +188,5 @@ extern char *crypt(); /* libufc */
 # define TERMIO struct sgttyb
 # define USE_SGTTY
 #endif
+
+#endif /* IDEFINES_H */
