@@ -18,7 +18,7 @@
 
 /* Some functions that didn't fit elsewhere */
 
-/* $Id: extra.c,v 1.5 2001/06/04 14:40:56 nmav Exp $ */
+/* $Id: extra.c,v 1.10 2001/09/09 09:54:30 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -37,7 +37,7 @@
 #include "mcrypt_int.h"
 
 static char rcsid[] =
-    "$Id: extra.c,v 1.5 2001/06/04 14:40:56 nmav Exp $";
+    "$Id: extra.c,v 1.10 2001/09/09 09:54:30 nmav Exp $";
 
 extern int double_check;
 extern char *outfile;
@@ -50,7 +50,7 @@ extern int hash_algorithm;
 char *my_getpass(char *prt)
 {
 	char *atmp;
-	char *ztmp = secure_mcrypt_malloc(MAX_KEY_LEN);
+	char *ztmp = _secure_mcrypt_malloc(MAX_KEY_LEN);
 	char *btmp;
 	char string_tmp[200];
 
@@ -61,7 +61,7 @@ char *my_getpass(char *prt)
 	strlcpy(ztmp, atmp, MAX_KEY_LEN);
 
 #ifndef NO_GETPASS
-	secure_mcrypt_free(atmp, strlen(atmp));
+	_secure_mcrypt_free(atmp, strlen(atmp));
 #else
 	Bzero(atmp, strlen(atmp));
 #endif
@@ -76,15 +76,15 @@ char *my_getpass(char *prt)
 			_
 			("Keywords do not match or they are too long.\n"));
 #ifndef NO_GETPASS
-		secure_mcrypt_free(btmp, MAX_KEY_LEN);
+		_secure_mcrypt_free(btmp, MAX_KEY_LEN);
 #else
 		Bzero(btmp, strlen(btmp));
 #endif
-		secure_mcrypt_free(ztmp, MAX_KEY_LEN);
+		_secure_mcrypt_free(ztmp, MAX_KEY_LEN);
 		return NULL;
 	}
 #ifndef NO_GETPASS
-	secure_mcrypt_free(btmp, MAX_KEY_LEN);
+	_secure_mcrypt_free(btmp, MAX_KEY_LEN);
 #else
 	Bzero(btmp, strlen(btmp));
 #endif
@@ -277,7 +277,7 @@ void *read_iv(FILE * fstream, int ivsize)
 		fstream = (FILE *) stdin;
 	}
 
-	IV = mcrypt_malloc(ivsize);
+	IV = _mcrypt_malloc(ivsize);
 	fread(IV, 1, ivsize, fstream);
 
 	return IV;
@@ -289,7 +289,7 @@ void _tolow(char *str, int size)
 	int i;
 
 	for (i = 0; i < size; i++) {
-		str[i] = _tolower(str[i]);
+		str[i] = tolower(str[i]);
 	}
 }
 
@@ -301,12 +301,9 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	short int keylen = *keysize;
 	unsigned char null = 0;
 	unsigned char sflag = 0;
-#ifdef LIBMCRYPT22
-	char* index;
-	char algo[70];
-#endif
+	char tmp[255];
 
-	buf = mcrypt_malloc(4);
+	buf = _mcrypt_malloc(4);
 
 	buf[0] = '\0';
 	buf[1] = 'm';
@@ -321,13 +318,13 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	if (fwrite(buf, 1, 4, filedes) != 4) {
 		return 1;
 	}
-	mcrypt_free(buf);
+	_mcrypt_free(buf);
 
 	if (fwrite(algorithm, 1, strlen(algorithm), filedes) !=
 	    strlen(algorithm)) {
 		return 1;
 	}
-	fwrite(&null, 1, 1, filedes);
+	if (fwrite(&null, 1, 1, filedes)!=1) return 1;
 
 #ifdef WORDS_BIGENDIAN
 	keylen = byteswap16(keylen);
@@ -340,13 +337,13 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	if (fwrite(mode, 1, strlen(mode), filedes) != strlen(mode)) {
 		return 1;
 	}
-	fwrite(&null, 1, 1, filedes);
+	if (fwrite(&null, 1, 1, filedes)!=1) return 1;
 
 	if (fwrite(keymode, 1, strlen(keymode), filedes) !=
 	    strlen(keymode)) {
 		return 1;
 	}
-	fwrite(&null, 1, 1, filedes);
+	if (fwrite(&null, 1, 1, filedes)!=1) return 1;
 
 	if (salt != NULL) {
 		sflag = salt_size;
@@ -360,12 +357,17 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 
 
 	buf = mhash_get_hash_name(hash_algorithm);
-	_tolow(buf, strlen(buf));
-	if (fwrite(buf, 1, strlen(buf), filedes) != strlen(buf)) {
+	if (buf!=NULL) {
+		strcpy(tmp, buf);
+		free(buf);
+		_tolow(tmp, strlen(tmp));
+	} else
+		return 1;
+		
+	if (fwrite(tmp, 1, strlen(tmp), filedes) != strlen(buf)) {
 		return 1;
 	}
-	mcrypt_free(buf);
-	fwrite(&null, 1, 1, filedes);
+	if (fwrite(&null, 1, 1, filedes)!=1) return 1;
 
 
 	return 0;
@@ -378,7 +380,7 @@ int write_iv(FILE * filedes, void *IV, int ivsize)
 	unsigned char *buf = NULL;
 
 	if (ivsize > 0) {
-		buf = mcrypt_malloc(ivsize);
+		buf = _mcrypt_malloc(ivsize);
 		if (IV != NULL) {
 			Bzero(buf, ivsize);
 			memmove(buf, IV, ivsize);
@@ -390,7 +392,7 @@ int write_iv(FILE * filedes, void *IV, int ivsize)
 		}
 	}
 
-	mcrypt_free(buf);
+	_mcrypt_free(buf);
 	return 0;
 
 }
@@ -490,8 +492,8 @@ char **read_key_file(char *file, int *num)
 	}
 	while (fgets(keyword, MAX_KEY_LEN, FROMF) != NULL) {
 		x++;
-		keys = mcrypt_realloc(keys, x * sizeof(char *));
-		keys[x - 1] = mcrypt_malloc(strlen(keyword) + 1);
+		keys = _mcrypt_realloc(keys, x * sizeof(char *));
+		keys[x - 1] = _mcrypt_malloc(strlen(keyword) + 1);
 /* Remove newline */
 		if (keyword[strlen(keyword) - 1] == '\n')
 			keyword[strlen(keyword) - 1] = '\0';
@@ -516,11 +518,11 @@ char *get_cfile(int uid, char *cfile)
 	pwd = getpwuid(uid);
 
 	if (pwd != NULL) {
-		home = mcrypt_malloc(strlen(pwd->pw_dir) + strlen(cfile) + 2);
+		home = _mcrypt_malloc(strlen(pwd->pw_dir) + strlen(cfile) + 2);
 		strcpy(home, pwd->pw_dir);
 		strcat(home, "/");
 	} else {
-		home = mcrypt_calloc(1, strlen(cfile) + 2);
+		home = _mcrypt_calloc(1, strlen(cfile) + 2);
 	}
 	strcat(home, cfile);
 
