@@ -57,7 +57,7 @@ return pass;
 
 extern int noecho;
 
-static char rcsid[] = "$Id: getpass.c,v 1.2 2001/06/04 14:40:56 nmav Exp $";
+static char rcsid[] = "$Id: getpass.c,v 1.3 2001/09/02 20:20:43 nmav Exp $";
 
 /* new code, #undef if there are any problems...  */
 #ifdef HAVE_SIGLONGJMP
@@ -87,7 +87,7 @@ char *
  readpass(FILE * fp)
 {
 	static char asterix[MAXLEN + 1];
-        char *input=secure_mcrypt_malloc(MAXLEN + 1);
+        char *input=_secure_mcrypt_malloc(MAXLEN + 1);
 	char *cp, *ap, c;
 	int i;
 
@@ -149,7 +149,7 @@ char *
 char *
  readpass_old(FILE * fp)
 {
-	char* input=secure_mcrypt_malloc(MAXLEN + 1);
+	char* input=_secure_mcrypt_malloc(MAXLEN + 1);
 	char *cp;
 
 	fflush(stderr);
