@@ -1,7 +1,7 @@
 /*  mcrypt 2.5 - encrypts text or binary files using symmetric algorithms.
  *  For a brief description of the algorithms read the man page.
  *
- *    Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
+ *    Copyright (C) 1998,1999,2000,2001 Nikos Mavroyanopoulos
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  */
 
 
-/* $Id: mcrypt.c,v 1.6 2001/06/04 14:40:56 nmav Exp $ */
+/* $Id: mcrypt.c,v 1.12 2001/09/11 08:09:41 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -34,10 +34,10 @@
 #include <gaa.h>
 #include <errors.h>
 #include <environ.h>
-
+#include <popen.h>
 
 static char rcsid[] =
-    "$Id: mcrypt.c,v 1.6 2001/06/04 14:40:56 nmav Exp $";
+    "$Id: mcrypt.c,v 1.12 2001/09/11 08:09:41 nmav Exp $";
 
 char tmperr[128];
 unsigned int stream_flag = FALSE;
@@ -149,7 +149,7 @@ void usage(void)
 void mcrypt_license()
 {
 	fprintf(stdout,
-		_("\nCopyright (C) 1998,1999,2000 Nikos Mavroyanopoulos\n"
+		_("\nCopyright (C) 1998-2001 Nikos Mavroyanopoulos\n"
 		  "This program is free software; you can redistribute it and/or modify \n"
 		  "it under the terms of the GNU General Public License as published by \n"
 		  "the Free Software Foundation; either version 2 of the License, or \n"
@@ -167,65 +167,26 @@ int check_hash_algo(char *chain)
 {
 	int i;
 	char *y;
-
+	char tmp[255];
+	
 	for (i = 0; i < 255; i++) {
 		y = mhash_get_hash_name(i);
 		if (y != NULL) {
-			_tolow(y, strlen(y));
-			if (strcmp(y, chain) == 0) {
-				mcrypt_free(y);
+			strcpy(tmp, y);
+			free(y);
+			_tolow(tmp, strlen(tmp));
+			if (strcmp(tmp, chain) == 0) {
 				return i;
 			}
-			mcrypt_free(y);
 		}
 	}
 	return -1;
 }
 
 
-#ifdef LIBMCRYPT22
-int check_algo(char *chain)
-{
-	int i;
-	char *y;
-
-	for (i = 0; i < 255; i++) {
-		if (mcrypt_is_ok_algorithm(i) == 0) {
-			y = mcrypt_get_algorithms_name(i);
-			if (strcmp(y, chain) == 0) {
-				mcrypt_free(y);
-				return i;
-			}
-			mcrypt_free(y);
-		}
-	}
-	return -1;
-}
-
-int check_mode(char *chain)
-{
-	int i;
-	char *y;
-
-	for (i = 0; i < 255; i++) {
-		if (mcrypt_is_ok_mode(i) == 0) {
-			y = mcrypt_get_modes_name(i);
-			if (strcmp(y, chain) == 0) {
-				mcrypt_free(y);
-				return i;
-			}
-			mcrypt_free(y);
-		}
-	}
-	return -1;
-}
-#endif
-
 #ifndef LIBMCRYPT_VERSION
 # ifdef LIBMCRYPT24
 #  define LIBMCRYPT_VERSION "2.4"
-# else
-#  define LIBMCRYPT_VERSION "2.2"
 # endif
 #endif
 
@@ -238,7 +199,7 @@ void mcrypt_version()
 		LIBMCRYPT_VERSION);
 	fprintf(stderr,
 		_
-		("Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos (nmav@hellug.gr)\n"));
+		("Copyright (C) 1998-2001 Nikos Mavroyanopoulos (nmav@hellug.gr)\n"));
 
 }
 
@@ -250,9 +211,6 @@ int main(int argc, char **argv)
 	char *einfile = 0, *dinfile = 0, *keyword = 0;
 	char **file, **keyfile = NULL, *cfile;
 	int x, y, i, file_count = 0, keys = 0, used_algo = FALSE;
-#ifndef LIBMCRYPT22
-	int *siz;
-#endif
 	gaainfo info;
 
 #ifdef HAVE_SIGNAL_H
@@ -277,7 +235,7 @@ int main(int argc, char **argv)
 		err_quit(_("Error in the arguments.\n"));
 	}
 	if (info.config_file != NULL) {
-		cfile = mcrypt_malloc(strlen(info.config_file));
+		cfile = _mcrypt_malloc(strlen(info.config_file));
 		strcpy(cfile, info.config_file);
 		if (check_file(cfile) != 1) {
 			err_crit(_
@@ -287,7 +245,7 @@ int main(int argc, char **argv)
 #ifdef HAVE_GETPWUID
 		cfile = get_cfile(getuid(), ".mcryptrc");
 #else
-		cfile = mcrypt_malloc(11);
+		cfile = _mcrypt_malloc(11);
 		strcpy(cfile, ".mcryptrc");
 #endif
 	}
@@ -299,22 +257,22 @@ int main(int argc, char **argv)
 			err_quit(_("Error in the configuration file.\n"));
 		}
 	}
-	mcrypt_free(cfile);
+	_mcrypt_free(cfile);
 
 
 	if (check_env() == TRUE) {
 
 		if (get_env_key() != NULL) {
 			info.keylen = 1;
-			mcrypt_free(info.keys);
+			_mcrypt_free(info.keys);
 			info.keys = get_env_key();
 		}
 		if (get_env_algo() != NULL) {
-			mcrypt_free(info.algorithm);
+			_mcrypt_free(info.algorithm);
 			info.algorithm = get_env_algo();
 		}
 		if (get_env_mode() != NULL) {
-			mcrypt_free(info.mode);
+			_mcrypt_free(info.mode);
 			info.mode = get_env_mode();
 		}
 		if (get_env_bit_mode() != 0) {
@@ -347,7 +305,7 @@ int main(int argc, char **argv)
 	/* File pointers are as much as the file arguments */
 
 	if ((info.size) * sizeof(char *) > 0) {
-		file = mcrypt_malloc((info.size) * sizeof(char *));
+		file = _mcrypt_malloc((info.size) * sizeof(char *));
 	} else {
 		file = NULL;
 	}
@@ -374,11 +332,8 @@ int main(int argc, char **argv)
 	double_check = info.double_check;
 	flush = info.flush;
 	nodelete = info.nodelete;
-#ifdef LIBMCRYPT22
-	keysize = 0;
-#else
 	keysize = info.keysize;
-#endif
+
 	algorithms_directory = info.algorithms_directory;
 	modes_directory = info.modes_directory;
 #ifdef ZIP
@@ -393,7 +348,7 @@ int main(int argc, char **argv)
 	}
 
 	if (info.hash != NULL) {
-		hash = mcrypt_malloc(strlen(info.hash));
+		hash = _mcrypt_malloc(strlen(info.hash));
 		strcpy(hash, info.hash);
 		hash_algorithm = check_hash_algo(hash);
 
@@ -410,7 +365,7 @@ int main(int argc, char **argv)
 		}
 	} else {
 		hash_algorithm = MHASH_CRC32;
-		hash = mcrypt_malloc(strlen("crc32") + 1);
+		hash = _mcrypt_malloc(strlen("crc32") + 1);
 		strcpy(hash, "crc32");
 	}
 
@@ -424,12 +379,12 @@ int main(int argc, char **argv)
 	} else {
 		if (info.keys != NULL) {
 			kin = 2;
-			keyfile = mcrypt_malloc(info.keylen * sizeof(char *));
+			keyfile = _mcrypt_malloc(info.keylen * sizeof(char *));
 			keys = info.keylen;
 
 			for (i = 0; i < info.keylen; i++) {
 				keyfile[i] =
-				    secure_mcrypt_malloc(strlen(info.keys[i]) +
+				    _secure_mcrypt_malloc(strlen(info.keys[i]) +
 						   10);
 				strcpy(keyfile[i], info.keys[i]);
 			}
@@ -437,7 +392,7 @@ int main(int argc, char **argv)
 	}
 
 	if (info.mode != NULL) {
-		mode = mcrypt_malloc(strlen(info.mode) + 1);
+		mode = _mcrypt_malloc(strlen(info.mode) + 1);
 		strcpy(mode, info.mode);
 	} else {
 		mode = strdup(DEFAULT_MODE);
@@ -445,11 +400,9 @@ int main(int argc, char **argv)
 	
 	if (info.algorithm != NULL) {
 		used_algo = TRUE;
-		algorithm = mcrypt_malloc(strlen(info.algorithm) + 1);
+		algorithm = _mcrypt_malloc(strlen(info.algorithm) + 1);
 		strcpy(algorithm, info.algorithm);
 
-#ifndef LIBMCRYPT22
-
 		i = mcrypt_module_is_block_algorithm(algorithm, algorithms_directory);
 
 		if (i < 0) {
@@ -462,7 +415,7 @@ int main(int argc, char **argv)
 			y = mcrypt_module_is_block_algorithm_mode(mode, modes_directory);
 			if (y >= 0) {
 				mode =
-				    mcrypt_realloc(mode, strlen("stream") + 1);
+				    _mcrypt_realloc(mode, strlen("stream") + 1);
 				strcpy(mode, "stream");
 			} else {
 				fprintf(stderr, _("Error in mode '%s'.\n"),
@@ -471,56 +424,21 @@ int main(int argc, char **argv)
 			}
 		}
 
-#else
-
-		i = check_algo(algorithm);
-		if (i < 0) {
-			fprintf(stderr, _("Error in algorithm '%s'.\n"),
-				algorithm);
-			exit(1);
-		}
-		y = check_mode(mode);
-		if (y < 0) {
-			fprintf(stderr, _("Error in mode '%s'.\n"), mode);
-			exit(1);
-		}
-		if (mcrypt_is_block_algorithm(i) == 0) {
-			if (mcrypt_is_block_algorithm_mode(y) != 0) {
-				mode =
-				    mcrypt_realloc(mode, strlen("stream") + 1);
-				strcpy(mode, "stream");
-			}
-		}
-#endif
 	} else {
 		algorithm = strdup(DEFAULT_ALGORITHM);
 	}
 
 	if (info.mode != NULL) {
-#ifndef LIBMCRYPT22
 		y = mcrypt_module_is_block_algorithm_mode(mode, modes_directory);
 		if (y < 0) {
 			fprintf(stderr, _("Error in mode '%s'.\n"), mode);
 			exit(1);
 		}
-#else
-		y = check_mode(mode);
-		if (y < 0) {
-			fprintf(stderr, _("Error in mode '%s'.\n"), mode);
-			exit(1);
-		}
-#endif
 	}
 
-#ifdef LIBMCRYPT22
-
-	i = check_algo(algorithm);
-	i = keysize = mcrypt_get_key_size(i);
-
-#endif
-
 	if (info.keysize != 0) {
-#ifndef LIBMCRYPT22
+		int *siz;
+		
 		if (info.keysize > mcrypt_module_get_algo_key_size(algorithm, algorithms_directory)) {
 			err_quit(_("The specified key size is too large for this algorithm.\n"));
 		}
@@ -537,9 +455,6 @@ int main(int argc, char **argv)
 			err_quit(_("The specified key size not valid for this algorithm.\n"));
 		}
 		free(siz);
-#else
-		err_warn(_("The key size parameter is ignored in that version of mcrypt.\n"));
-#endif
 
 	}
 
@@ -587,7 +502,7 @@ int main(int argc, char **argv)
 
 		if (i != 0) {
 			if (outfile != NULL)
-				mcrypt_free(outfile);
+				_mcrypt_free(outfile);
 		}
 
 		/* If keyword file specified choose the i-th keyword */
@@ -595,7 +510,7 @@ int main(int argc, char **argv)
 		if (kin == 2 && i <= (keys - 1)) {
 			keyword = keyfile[i];
 			if (i != 0) {
-				secure_mcrypt_free(keyfile[i - 1],
+				_secure_mcrypt_free(keyfile[i - 1],
 					     strlen(keyfile[i - 1]));	/* Free previous keyword */
 			}
 		}
@@ -613,14 +528,6 @@ int main(int argc, char **argv)
 		}
 #endif
 
-#ifdef ZIP
-		if (stream_flag == TRUE) {
-			if (gzipflag == TRUE || bzipflag == TRUE) {
-				err_quit(_
-					 ("--bzip2 or --gzip parameters cannot be used when dealing with streams.\n"));
-			}
-		}
-#endif
 
 		/* Check how we were called */
 		if (din == TRUE) {	/* decryption */
@@ -664,7 +571,7 @@ int main(int argc, char **argv)
 				     && dinfile[y - 2] == 'n'
 				     && dinfile[y - 3] == '.')) {
 
-					outfile = mcrypt_calloc(y - 2, 1);
+					outfile = _mcrypt_calloc(y - 2, 1);
 					strncpy(outfile, dinfile, y - 3);
 #ifdef HAVE_STAT
 					/* But if it exists exit */
@@ -684,7 +591,7 @@ int main(int argc, char **argv)
 				} else {	/* append .dc */
 
 					cleanDelete = TRUE;
-					outfile = mcrypt_calloc(y + 5, 1);
+					outfile = _mcrypt_calloc(y + 5, 1);
 					strncpy(outfile, dinfile, y);
 					strcat(outfile, ".dc");
 
@@ -692,7 +599,7 @@ int main(int argc, char **argv)
 				}
 			} else {	/* encryption- append .nc */
 				outfile =
-				    mcrypt_calloc(strlen(einfile) + 5 + 4, 1);
+				    _mcrypt_calloc(strlen(einfile) + 5 + 4, 1);
 				strcpy(outfile, einfile);
 				/* if file has already the .nc ignore it */
 				if (strstr(outfile, ".nc") != NULL) {
@@ -730,7 +637,7 @@ int main(int argc, char **argv)
 
 			}
 		} else {	/* if streams */
-			outfile = mcrypt_malloc(7);
+			outfile = _mcrypt_malloc(7);
 			strcpy(outfile, "stdout");
 		}
 
@@ -849,7 +756,7 @@ int main(int argc, char **argv)
 /* Clear the last keyword used */
 	if (keyword != NULL) {
 		Bzero(keyword, strlen(keyword));
-		mcrypt_free(keyword);
+		_mcrypt_free(keyword);
 	}
 
 /* Clear ALL arguments in the command line */
@@ -886,13 +793,9 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	int full_blocks = 0, rest_blocks = 0;
 	char *fcrc32 = NULL;
 	word32 *keyword;
-#ifdef LIBMCRYPT22
-	int algo, _mode;
-#else
 	int tc;
-#endif
 	MHASH tm = 0;
-	int i = 0, how = 0;
+	int i = 0, how = 0, clen;
 	unsigned int len = 0;
 	FILE *FROMF;
 	int keylen, salt_size;
@@ -908,7 +811,6 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	word8 *ciphertext, *ciphertext_pad = NULL;
 	FILE *TOF = NULL;
 
-#ifndef LIBMCRYPT22
 	td =
 	    mcrypt_module_open(algorithm, algorithms_directory, mode,
 			       modes_directory);
@@ -917,35 +819,17 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		err_crit(_("Mcrypt failed to open module.\n"));
 		return 1;
 	}
-#else
-	algo = check_algo(algorithm);
-	_mode = check_mode(mode);
-	if (algo < 0 || _mode < 0) {
-		err_crit(_("Mcrypt failed to test algorithms and modes.\n"));
-		return 1;
-	}
 
-#endif
-
-#ifndef LIBMCRYPT22
 	if (keysize == 0)
 		keysize = mcrypt_enc_get_key_size(td);
 
 	keylen = keysize;
 	blocksize = mcrypt_enc_get_block_size(td);
-#else
-	if (keysize == 0)
-		keysize = mcrypt_get_key_size(algo);
 
-	keylen = keysize;
-	blocksize = mcrypt_get_block_size(algo);
-#endif
-
-
-	ciphertext = mcrypt_malloc(blocksize * BUFFER_SIZE);
+	ciphertext = _mcrypt_malloc(blocksize * BUFFER_SIZE);
 
 	if (bare_flag == FALSE) {
-		salt = mcrypt_malloc(SALT_SIZE);	/* 20 bytes salt */
+		salt = _mcrypt_malloc(SALT_SIZE);	/* 20 bytes salt */
 
 		/* Fill the salt with random data */
 		for (i = 0; i < SALT_SIZE / sizeof(word32); i++) {
@@ -957,15 +841,9 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 			}
 		}
 	}
-#ifndef LIBMCRYPT22
-	IV = secure_mcrypt_malloc(mcrypt_enc_get_iv_size(td));
+	IV = _secure_mcrypt_malloc(mcrypt_enc_get_iv_size(td));
 /* Random number generator */
 	for (i = 0; i < mcrypt_enc_get_iv_size(td) / sizeof(word32); i++) {
-#else
-	IV = secure_mcrypt_malloc(mcrypt_get_block_size(algo));
-/* Random number generator */
-	for (i = 0; i < mcrypt_get_block_size(algo) / sizeof(word32); i++) {
-#endif
 		if (noiv == FALSE) {
 			if (real_random_flag == TRUE) {
 				IV[i] = SRAND32;
@@ -986,7 +864,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 #endif
 
 /* open files */
-	if (stream_flag == TRUE) {
+	if (stream_flag == TRUE && (gzipflag==FALSE && bzipflag==FALSE)) {
 		FROMF = stdin;
 		TOF = stdout;
 	} else {
@@ -1015,32 +893,42 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 #ifdef ZIP
 		} else {
 			/* command if using gzip */
+			clen = 5;
+			if (fromfile!=NULL) clen+=strlen(fromfile);
+
 			if (gzipflag == TRUE) {
+				clen+=strlen(GZIP);
 				command =
-				    malloc(strlen(GZIP) +
-					   strlen(fromfile) + 5);
+				    _mcrypt_malloc(clen);
 				strcpy(command, GZIP);
 			}
 			if (bzipflag == TRUE) {
+				clen+=strlen(BZIP2);
 				command =
-				    malloc(strlen(BZIP2) +
-					   strlen(fromfile) + 5);
+				    _mcrypt_malloc(clen);
 				strcpy(command, BZIP2);
 			}
 			strcat(command, " -c ");
-			strcat(command, fromfile);
-			FROMF = popen(command, "r");
+			if (fromfile!=NULL)
+				strcat(command, fromfile);
+			
+			if (stream_flag==FALSE) {
+				FROMF = popen(command, "r");
+				TOF = fopen(tofile, "wb");
+			} else { /* stream_flag==TRUE */
+				if (ppopen( command, NULL, &FROMF) <= 0) {
+					err_crit("ppopen error\n");
+					return -1;
+				}
+				TOF = stdout;
+			}
 
-			if (FROMF == NULL) {
-				perror("popen");
+			if (FROMF == NULL || TOF==NULL) {
+				perror("ppopen");
 				return 1;
 			}
-			TOF = fopen(tofile, "wb");
-			if (TOF == NULL) {
-				perror("fopen");
-				return -1;
-			}
-			if (write_lock(fileno((FILE *) TOF)) == -1) {
+
+			if (stream_flag==FALSE && write_lock(fileno((FILE *) TOF)) == -1) {
 				err_crit("Unable to lock file. Try --nolock\n");
 				return -1;
 			}
@@ -1064,18 +952,21 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	}
 
 	if (bare_flag == FALSE) {
-		write_file_head(TOF, algorithm, mode, keymode, &keysize,
-				salt, salt_size);
+		if (write_file_head(TOF, algorithm, mode, keymode, &keysize,
+				salt, salt_size)!=0) {
+			err_crit("Error writing file\n");
+			return -1;		
+		}
 	}
-#ifndef LIBMCRYPT22
+
 	if (mcrypt_enc_mode_has_iv(td) == 1 && noiv == FALSE) {
-		write_iv(TOF, IV, mcrypt_enc_get_iv_size(td));
+		if (write_iv(TOF, IV, mcrypt_enc_get_iv_size(td))!=0) {
+			err_crit("Error writing file\n");
+			return -1;		
+		}		
 	}
-#else
-	if (mcrypt_mode_has_iv(_mode) == 1 && noiv == FALSE) {
-		write_iv(TOF, IV, mcrypt_get_block_size(algo));
-	}
-#endif
+
+	if (flush==TRUE) fflush(TOF);
 
 	if (quiet == FALSE)
 		fprintf(stderr,
@@ -1107,19 +998,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 
 
-#ifndef LIBMCRYPT22
 	tc = mcrypt_generic_init(td, keyword, len, IV);
 	if (tc < 0) {
 		mcrypt_perror(tc);
 		return 1;
 	}
-#else
-	td = mcrypt_generic_init(_mode, algo, keyword, len, IV);
-	if (td == MCRYPT_FAILED) {
-		err_crit("There was an error in mcrypt initialization\n");
-		return 1;
-	}
-#endif
 
 	if (bare_flag == FALSE)
 		tm = mhash_init(hash_algorithm);
@@ -1138,11 +1021,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 			if (bare_flag == FALSE)
 				mhash(tm, ciphertext, how);
 
-#ifndef LIBMCRYPT22
 			if (mcrypt_enc_is_block_mode(td) == 1) {
-#else
-			if (mcrypt_is_block_mode(_mode) == 1) {
-#endif
 				rest_blocks = how % blocksize;
 				full_blocks = how / blocksize;
 
@@ -1188,32 +1067,20 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		    make_mult(rest_blocks + hash_size + 1, blocksize);
 
 		/* in case of a stream cipher no padding is needed */
-#ifndef LIBMCRYPT22
 		if (mcrypt_enc_is_block_mode(td) == 0) {
-#else
-		if (mcrypt_is_block_mode(_mode) == 0) {
-#endif
 			pad_size = hash_size;
 		}
 	} else {		/* bare_flag==TRUE */
 		pad_size = blocksize;
-#ifndef LIBMCRYPT22
 		if (mcrypt_enc_is_block_mode(td) == 0) {
-#else
-		if (mcrypt_is_block_mode(_mode) == 0) {
-#endif
 			pad_size = 0;
 		}
 	}
 
 	if (pad_size > 0)
-		ciphertext_pad = mcrypt_malloc(pad_size);
+		ciphertext_pad = _mcrypt_malloc(pad_size);
 
-#ifndef LIBMCRYPT22
 	if (mcrypt_enc_is_block_mode(td) == 1) {
-#else
-	if (mcrypt_is_block_mode(_mode) == 1) {
-#endif
 		memmove(ciphertext_pad,
 			&ciphertext[full_blocks * blocksize], rest_blocks);
 		if (bare_flag == FALSE) {
@@ -1248,7 +1115,6 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	/* End of copy */
 	/* close and unlock files */
 	fflush(TOF);
-	unlock(fileno((FILE *) TOF));
 
 	if (stream_flag == FALSE) {
 		fflush(FROMF);
@@ -1276,11 +1142,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	mcrypt_generic_end(td);
 
 	if (bare_flag == FALSE) {
-		mcrypt_free(salt);
+		_mcrypt_free(salt);
 	}
 
-	secure_mcrypt_free(keyword, keylen);
-	mcrypt_free(ciphertext);
+	_secure_mcrypt_free(keyword, keylen);
+	_mcrypt_free(ciphertext);
 
 
 	return 0;
@@ -1306,17 +1172,13 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	word8 *ciphertext_old, *ciphertext_p_old;
 	word32 *keyword;
 	word32 *salt;
-	char local_algorithm[50];
+	char local_algorithm[50], *command;
 	char local_mode[50];
 	char local_keymode[50];
 	char local_salt[100];
 	unsigned int len = 0;
-	int hash_size, pad_size;
-#ifdef LIBMCRYPT22
-	int algo, _mode;
-#else
+	int hash_size, pad_size, clen;
 	int j;
-#endif
 	FILE *RTOF;
 	FILE *FROMF;
 
@@ -1325,6 +1187,35 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	if (stream_flag == TRUE) {
 		FROMF = stdin;
 		RTOF = stdout;
+#ifdef ZIP
+		if (bzipflag==TRUE || gzipflag==TRUE) {
+			clen = 10;
+			if (fromfile!=NULL) clen+=strlen(fromfile);
+
+			if (gzipflag == TRUE) {
+				clen+=strlen(GZIP);
+				command =
+				    _mcrypt_malloc(clen);
+				strcpy(command, GZIP);
+			}
+			if (bzipflag == TRUE) {
+				clen+=strlen(BZIP2);
+				command =
+				    _mcrypt_malloc(clen);
+				strcpy(command, BZIP2);
+			}
+			strcat(command, " -c -d ");
+			if (fromfile!=NULL)
+				strcat(command, fromfile);
+			
+			ppopen( command, &RTOF, NULL);
+		
+			if (RTOF==NULL) {
+				err_crit("ppopen error\n");
+				return -1;
+			}
+		}
+#endif
 	} else {
 		FROMF = fopen(fromfile, "rb");
 		if (FROMF == NULL) {
@@ -1350,20 +1241,20 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 
 	if (bare_flag == FALSE) {
-		salt = mcrypt_malloc(SALT_SIZE);
+		salt = _mcrypt_malloc(SALT_SIZE);
 		if (check_file_head
 		    (FROMF, local_algorithm, local_mode, local_keymode,
 		     &keysize, local_salt, &salt_size) != 0) {
 			err_crit(_("No valid file headers found.\n"));
 			return 1;
 		}
-		salt = mcrypt_malloc(salt_size);
+		salt = _mcrypt_malloc(salt_size);
 		memmove(salt, local_salt, salt_size);
-		algorithm = mcrypt_malloc(strlen(local_algorithm)+1);
+		algorithm = _mcrypt_malloc(strlen(local_algorithm)+1);
 		strcpy(algorithm, local_algorithm);
-		mode = mcrypt_malloc(strlen(local_mode)+1);
+		mode = _mcrypt_malloc(strlen(local_mode)+1);
 		strcpy(mode, local_mode);
-		keymode = mcrypt_malloc(strlen(local_keymode)+1);
+		keymode = _mcrypt_malloc(strlen(local_keymode)+1);
 		strcpy(keymode, local_keymode);
 	} else {
 		salt = NULL;
@@ -1371,20 +1262,6 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 	hash_size = mhash_get_block_size(hash_algorithm);
 
-#ifdef LIBMCRYPT22
-	algo = check_algo(algorithm);
-	_mode = check_mode(mode);
-	if (algo < 0 || _mode < 0) {
-		err_crit("Unable to test algorithms and modes.\n");
-		return 1;
-	}
-
-	if (keysize == 0)
-		keysize = mcrypt_get_key_size(algo);
-
-	blocksize = mcrypt_get_block_size(algo);
-
-#else
 	td =
 	    mcrypt_module_open(algorithm, algorithms_directory, mode,
 			       modes_directory);
@@ -1399,10 +1276,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 	blocksize = mcrypt_enc_get_block_size(td);
 
-#endif
-
-	ciphertext = mcrypt_malloc(blocksize * BUFFER_SIZE);
-	ciphertext_old = mcrypt_malloc(blocksize);
+	ciphertext = _mcrypt_malloc(blocksize * BUFFER_SIZE);
+	ciphertext_old = _mcrypt_malloc(blocksize);
 
 	if (quiet == FALSE)
 		fprintf(stderr,
@@ -1433,27 +1308,16 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		return 1;
 	}
 
-#ifndef LIBMCRYPT22
 	if (mcrypt_enc_mode_has_iv(td) == 1) {
 		if (noiv == FALSE) {
 			IV = read_iv(FROMF, mcrypt_enc_get_iv_size(td));
 		} else {
-			IV = mcrypt_calloc(1, mcrypt_enc_get_iv_size(td));
+			IV = _mcrypt_calloc(1, mcrypt_enc_get_iv_size(td));
 		}
 	} else {
 		IV = NULL;
 	}
-#else
-	if (mcrypt_mode_has_iv(_mode) == 1) {
-		if (noiv == FALSE) {
-			IV = read_iv(FROMF, mcrypt_get_block_size(algo));
-		} else {
-			IV = mcrypt_calloc(1, mcrypt_get_block_size(algo));
-		}
-	} else {
-		IV = NULL;
-	}
-#endif
+
 #ifdef DEBUG
 	fprintf(stderr, "IV: ");
 	for (i = 0; i < mcrypt_get_iv_size(td); i++) {
@@ -1463,26 +1327,18 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 #endif
 
 
-#ifndef LIBMCRYPT22
 	j = mcrypt_generic_init(td, keyword, len, IV);
 	if (j < 0) {
 		mcrypt_perror(j);
 		return 1;
 	}
-#else
-	td = mcrypt_generic_init(_mode, algo, keyword, len, IV);
-	if (td == MCRYPT_FAILED) {
-		err_crit("There was an error in mcrypt initialization\n");
-		return 1;
-	}
-#endif
 
 	if (bare_flag == FALSE) {
 		tm = mhash_init(hash_algorithm);
 	}
 
 	crcsize = hash_size;
-	fcrc32 = mcrypt_malloc(crcsize);
+	fcrc32 = _mcrypt_malloc(crcsize);
 
 	/* decryption starts here */
 
@@ -1490,25 +1346,17 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		pad_size = make_mult(blocksize + hash_size, blocksize);
 
 		/* in case of a stream cipher no padding is needed */
-#ifndef LIBMCRYPT22
 		if (mcrypt_enc_is_block_mode(td) == 0) {
-#else
-                if (mcrypt_is_block_mode(_mode) == 0) {
-#endif
 			pad_size = hash_size;
 		}
 
-		ciphertext_p_old = mcrypt_malloc(pad_size);
+		ciphertext_p_old = _mcrypt_malloc(pad_size);
 	} else {
 		pad_size = 0;
 		ciphertext_p_old = NULL;
 	}
 
-#ifndef LIBMCRYPT22
 	if (mcrypt_enc_is_block_mode(td) == 1) {
-#else
-	if (mcrypt_is_block_mode(_mode) == 1) {
-#endif
 		if (bare_flag == FALSE) {
 			how =
 			    fread(ciphertext, 1, blocksize * BUFFER_SIZE,
@@ -1569,7 +1417,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 					} else {
 						fprintf(stderr,
 							_
-							("Unexpected error\n"),
+							("Unexpected error [%d]\n"),
 							how);
 						return 1;
 					}
@@ -1824,24 +1672,20 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 	if (pad_size > 0) {
 		Bzero(ciphertext_p_old, pad_size);
-		mcrypt_free(ciphertext_p_old);
+		_mcrypt_free(ciphertext_p_old);
 	}
 	Bzero(ciphertext_old, blocksize);
 	Bzero(ciphertext, blocksize * BUFFER_SIZE);
 
-	mcrypt_free(ciphertext_old);
-	mcrypt_free(ciphertext);
+	_mcrypt_free(ciphertext_old);
+	_mcrypt_free(ciphertext);
 
 	Bzero(tmp_buf, BUFFER_SIZE);
-	secure_mcrypt_free(keyword, keysize);
+	_secure_mcrypt_free(keyword, keysize);
 
 	if (bare_flag == FALSE) {
 		newcrc32 = mhash_end(tm);
-#ifndef LIBMCRYPT22
 		if (mcrypt_enc_is_block_mode(td) == 1) {
-#else
-                if (mcrypt_is_block_mode(_mode) == 1) {
-#endif
 			if (hash_size > blocksize) {
 				hash_size = blocksize;
 			}
@@ -1895,8 +1739,10 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 			/* crc check passed */
 
 		} else {
+			command = mhash_get_hash_name(hash_algorithm);
 			fprintf(stderr, _("%s check failed\n"),
-				mhash_get_hash_name(hash_algorithm));
+				command);
+			free(command);
 			return 1;	/* CRC32s Do not match */
 		}
 	}
@@ -1925,7 +1771,6 @@ void rol_buf(void *buffer, int buffersize, void *value)
 	buf[buffersize - 1] = val[0];
 }
 
-#ifndef LIBMCRYPT22
 int print_list(void)
 {
 	MCRYPT td;
@@ -1937,8 +1782,20 @@ int print_list(void)
 
 
 	names = mcrypt_list_algorithms(algorithms_directory, &jmax);
+	if (names==NULL) {
+		if (algorithms_directory==NULL)
+			algorithms_directory = "default directory";
+		fprintf(stderr,"Could not find algorithms in %s\n", algorithms_directory);
+		exit(1);
+	}
 	modes = mcrypt_list_modes(modes_directory, &imax);
-
+	if (modes==NULL) {
+		if (modes_directory==NULL)
+			modes_directory = "default directory";
+		fprintf(stderr, "Could not find modes in %s\n", modes_directory);
+		exit(1);
+	}
+	
 	for (j = 0; j < jmax; j++) {
 		printf("%s ", names[j]);
 		start = 0;
@@ -1965,59 +1822,6 @@ int print_list(void)
 	mcrypt_free_p(modes, imax);
 	return 0;
 }
-#else
-int print_list(void)
-{
-	int tmpi, tmpj;
-	char *tmpc;
-
-	fprintf(stdout, _("Supported Algorithms and modes:\n\n"));
-	for (tmpi = 0; tmpi < 256; tmpi++) {
-		tmpc = mcrypt_get_algorithms_name(tmpi);
-		if (tmpc != NULL) {
-			fprintf(stdout, " %s:", tmpc);
-			mcrypt_free(tmpc);
-
-			if (mcrypt_is_block_algorithm(tmpi) == TRUE) {
-				for (tmpj = 0; tmpj < 256; tmpj++) {
-					if (mcrypt_is_block_algorithm_mode(tmpj) != 0) {
-						tmpc =
-						    mcrypt_get_modes_name
-						    (tmpj);
-						if (tmpc != NULL) {
-							fprintf(stdout,
-								" %s",
-								tmpc);
-							mcrypt_free(tmpc);
-						}
-					}
-				}
-				fprintf(stdout, "\n");
-			} else {
-				for (tmpj = 0; tmpj < 256; tmpj++) {
-					if (mcrypt_is_block_algorithm_mode(tmpj) == 0) {
-						tmpc =
-						    mcrypt_get_modes_name
-						    (tmpj);
-						if (tmpc != NULL) {
-							fprintf(stdout,
-								" %s",
-								tmpc);
-							mcrypt_free(tmpc);
-						}
-					}
-				}
-				fprintf(stdout, "\n");
-			}
-
-		}
-	}
-
-	return 0;
-}
-
-
-#endif
 
 int print_keylist(void)
 {
@@ -2042,14 +1846,16 @@ int print_hashlist(void)
 {
 	int tmpi;
 	char *tmpc;
+	char tmp[255];
 
 	fprintf(stdout, _("Supported Hash Algorithms:\n"));
 	for (tmpi = 0; tmpi < 256; tmpi++) {
 		tmpc = mhash_get_hash_name(tmpi);
 		if (tmpc != NULL) {
-			_tolow(tmpc, strlen(tmpc));
-			fprintf(stdout, "%s\n", tmpc);
-			mcrypt_free(tmpc);
+			strcpy( tmp, tmpc);
+			free(tmpc);
+			_tolow(tmp, strlen(tmp));
+			fprintf(stdout, "%s\n", tmp);
 		}
 	}
 	return 0;
